// ignore_for_file: public_member_api_docs, always_use_package_imports, non_constant_identifier_names, lines_longer_than_80_chars, avoid_positional_boolean_parameters, unused_element, use_setters_to_change_properties

import 'package:healthrian_common_support/healthrian_common_support.dart';

class DataFilter {
  DataFilter(int channelCount) {
    /// Initialize filter

    _filters = {
      for (final impulseType in ImpulseType.values)
        impulseType: {
          for (final samplingRate in SamplingRate.values)
            samplingRate: {
              for (final filterType in filterTypes)
                filterType: List.generate(
                  channelCount,
                  (_) => FilterSelector.construct(
                    impulseType: impulseType,
                    samplingRate: samplingRate,
                    filterType: filterType,
                  ),
                ),
            },
        },
    };
  }

  ///
  ImpulseType impulseType = ImpulseType.iir;
  SamplingRate samplingRate = SamplingRate.freq250hz;

  ///
  DcBlock dcBlock = DcBlock.on;
  LowPass lowPass = LowPass.lpf40;
  HighPass highPass = HighPass.hpf01;
  BandStop bandStop = BandStop.bsf60;
  Adaptive adaptive = Adaptive.off;

  late final Map<ImpulseType,
      Map<SamplingRate, Map<Enum, List<FilterInterface?>>>> _filters;

  double applyFilter(final int channel, double value,
      [final bool leadState = true]) {
    if (!leadState) return value;

    for (final filterType in [dcBlock, lowPass, highPass, bandStop, adaptive]) {
      final c = _filters[impulseType]?[samplingRate]?[filterType]?[channel]
          ?.apply(value);
      if (c != null) value = c;
    }
    return value;
  }

  int delaySample([final int channel = 0]) {
    var count = 0;
    for (final filterType in [dcBlock, lowPass, highPass, bandStop, adaptive]) {
      final delay =
          _filters[impulseType]?[samplingRate]?[filterType]?[channel]?.delay;
      count += (delay ?? 0);
    }
    return count;
  }
}
