import 'dart:developer' as dev;

import 'package:flutter/services.dart';
import 'package:healthrian_common_support/healthrian_common_support.dart';
import 'package:mobx/mobx.dart';

final filter = FilterManager();

class FilterManager {
  static final FilterManager _instance = FilterManager._internal();

  factory FilterManager() {
    return _instance;
  }

  FilterManager._internal() {
    initializeFileBasedFilter();
  }

  Future<void> initializeFileBasedFilter() async {
    fileData = {
      for (final samplingRate in SamplingRate.values)
        samplingRate: {
          for (final key in fileNameMap[samplingRate]!.keys)
            if (fileNameMap[samplingRate]?[key] != null)
              key: (await parseData(fileNameMap[samplingRate]![key]!)),
        }
    };
    await _done();
  }

  late final _done = Action(() => _completed.value = true);

  late final Map<SamplingRate, Map<Enum, List<double>>> fileData;

  final _completed = Observable(false);

  Future<void> waitForCompletion() async {
    await asyncWhen((_) => _completed.value == true);
    dev.log('Completed $runtimeType');
  }

  Future<String> readString(String fileName) async {
    return await rootBundle
        .loadString('packages/healthrian_common_support/filter_data/$fileName');
  }

  Future<List<double>> parseData(String fileName) async {
    final text = await readString(fileName);
    final result = <String, List<double>>{};
    String? key;
    for (final line in text.split('\n')) {
      final trim = line.trim();
      if (trim.startsWith('%')) continue;
      if (trim.isEmpty) continue;

      if (trim.trim().endsWith(':')) {
        key = trim.substring(0, trim.length - 1);
        result[key] = [];
      } else if (key != null) {
        result[key]?.add(double.tryParse(trim) ?? 0);
      }
    }
    return result['Numerator']!;
  }
}
