import 'package:healthrian_common_support/healthrian_common_support.dart';

typedef FnFilterType = FilterInterface? Function(Enum filterType);
typedef FnSamplingRate = FnFilterType Function(SamplingRate samplingRate);
typedef FnImpulseType = FnSamplingRate Function(ImpulseType impulseType);

class FilterSelector {
  static FilterInterface? construct({
    required ImpulseType impulseType,
    required SamplingRate samplingRate,
    required Enum filterType,
  }) =>
      _selector(impulseType)(samplingRate)(filterType);

  static FnSamplingRate _selector(ImpulseType impulseType) {
    switch (impulseType) {
      case ImpulseType.iir:
        return _iir;
      case ImpulseType.fir:
        return _fir;
    }
  }

  static FnFilterType _iir(SamplingRate samplingRate) {
    switch (samplingRate) {
      case SamplingRate.freq250hz:
        return _freq250iir;
      case SamplingRate.freq500hz:
        return _freq500iir;
    }
  }

  static FnFilterType _fir(SamplingRate samplingRate) {
    switch (samplingRate) {
      case SamplingRate.freq250hz:
        return _freq250fir;
      case SamplingRate.freq500hz:
        return _freq500fir;
    }
  }

  static FilterInterface? _freq250iir(Enum filterType) {
    if (filterType == Adaptive.on) return AdaptiveFilter(is250Hz: true);
    if (filterType == DcBlock.on) return DcNotch();
    final data = IIRData.freq250hz[filterType];
    if (data == null) return null;
    final numerator = [...data.numerator];
    final denominator = [...data.denominator];
    return IIRFilter(numerator: numerator, denominator: denominator);
  }

  static FilterInterface? _freq500iir(Enum filterType) {
    if (filterType == Adaptive.on) return AdaptiveFilter(is250Hz: false);
    if (filterType == DcBlock.on) return null;
    final data = IIRData.freq500hz[filterType];
    if (data == null) return null;
    final numerator = [...data.numerator];
    final denominator = [...data.denominator];
    return IIRFilter(numerator: numerator, denominator: denominator);
  }

  static FilterInterface? _freq250fir(Enum filterType) {
    if (filterType == Adaptive.on) return AdaptiveFilter(is250Hz: true);
    if (filterType == DcBlock.on) return DcNotch();
    if (filterType == DcBlock.on) return null;
    final data = FIRData.freq250hz[filterType];
    if (data == null) return null;
    final impulseResponse = [...data];
    return FIRFilter(impulseResponse: impulseResponse);
  }

  static FilterInterface? _freq500fir(Enum filterType) {
    if (filterType == Adaptive.on) return AdaptiveFilter(is250Hz: false);
    if (filterType == DcBlock.on) return null;
    final data = FIRData.freq500hz[filterType];
    if (data == null) return null;
    final impulseResponse = [...data];
    return FIRFilter(impulseResponse: impulseResponse);
  }
}
